-- HCSW AI Labs
--   (C) 2004 Doug Hoyte and Hardcore Software
--
-- Jamaican 2 Player Dominoes Definition


module DominoDef where

  import Array
  import Random


  data Player = A | B | Nobody
    deriving (Eq, Show)

  type Move = (Int, Int, Int)

  type Board = ([(Int, Int)], [(Int, Int)], Int, Int, Player)



  tb :: Board
  tb = ([(1,1), (1,2), (2,3)], [(3,4),(3,5)], 5, 5, A)



  getTurn :: Board -> Player
  getTurn (a, b, s1, s2, p) = p


  createBoard :: IO (Board)
  createBoard = do (a, b) <- getHands [] [] (domList 6 0)
                   --return (a, b, -1, -1, A)
                   return (playDouble6s a b)

    where

      playDouble6s a b = if (elem (6,6) a) then
                           (filter (/= (6,6)) a, b, 6, 6, B)
                         else
                           (a, filter (/= (6,6)) b, 6, 6, A)


      getHands :: [(Int, Int)] -> [(Int, Int)] -> [(Int, Int)] -> IO ([(Int, Int)], [(Int, Int)])
      getHands a b [] = do return (a, b)
      getHands a b dl = do r1 <- randomRIO (1, length(dl))
                           v1 <- do return (dl !! (r1 - 1))
                           nl <- do return ((take (r1 - 1) dl) ++ (drop r1 dl))
                           r2 <- randomRIO (1, length(nl))
                           v2 <- do return (nl !! (r2 - 1))
                           ll <- do return ((take (r2 - 1) nl) ++ (drop r2 nl))
                           f <- getHands (v1:a) (v2:b) ll
                           return (f)

      domList :: Int -> Int -> [(Int, Int)]
      domList a b = if (a == -1) then
                      []
                    else if (b == a) then
                      (a,b) : (domList (a-1) 0)
                    else
                      (a,b) : (domList a (b+1))




  makeMove :: Board -> Move -> Board
  makeMove (a, b, s1, s2, A) (-1, -1, -1) = (a, b, s1, s2, B)
  makeMove (a, b, s1, s2, B) (-1, -1, -1) = (a, b, s1, s2, A)
  makeMove (a, b, s1, s2, p) (d1, d2, s) = if p == A then
                                             (filter (/= (d1,d2)) a, b, news1, news2, B)
                                           else
                                             (a, filter (/= (d1,d2)) b, news1, news2, A)

    where news1 = if (s == 1) then
                    if (d1 == s1) then d2 else d1
                  else
                    s1
          news2 = if (s == 2) then
                    if (d1 == s2) then d2 else d1
                  else
                    s2



  getMoves :: Board -> [Move]
  getMoves board = if (isAnyoneOut board /= Nobody) then
                     []
                   else
                     if getMovesAux board == [] then [(-1,-1,-1)]
                     else getMovesAux board

    where getMovesAux (a, b, s1, s2, A) = (filter (/= (-1,-1,-1)) (map (isValid s1 s2) a))
          getMovesAux (a, b, s1, s2, B) = (filter (/= (-1,-1,-1)) (map (isValid s1 s2) b))

          isValid :: Int -> Int -> (Int, Int) -> (Int, Int, Int)
          isValid s1 s2 (d1, d2) = if (s1 == d1 || s1 == d2) then
                                     (d1, d2, 1)
                                   else if (s2 == d1 || s2 == d2) then
                                     (d1, d2, 2)
                                   else
                                     (-1, -1, -1)


  whoWon :: Board -> Player
  whoWon board@(a,b,s1,s2,p) = if out /= Nobody then out
                               else if amoves == [(-1,-1,-1)] && bmoves == [(-1,-1,-1)] then
                                 if (dSum a) > (dSum b) then B
                                 else A
                               else Nobody

    where amoves = if p == A then getMoves board
                   else getMoves (makeMove board (-1,-1,-1))
          bmoves = if p == B then getMoves board
                   else getMoves (makeMove board (-1,-1,-1))
          out = isAnyoneOut board



  staticEval :: Board -> Player -> Int
  staticEval board@(a, b, s1, s2, p) A = if whoWon board == A then 100000
                                         else if whoWon board == B then -100000
                                         else (13 * (length b)) + (dSum b) - (13 * (length a)) - (dSum a)
  staticEval board B = 0 - (staticEval board A)



  isAnyoneOut :: Board -> Player
  isAnyoneOut ([], b, s1, s2, p) = A
  isAnyoneOut (a, [], s1, s2, p) = B
  isAnyoneOut (a,  b, s1, s2, p) = Nobody


  dSum [] = 0
  dSum ((x1,x2):xs) = (x1+x2) + (dSum xs)
