module GameAI (GameTree, createGameTree, getBestMove) where

  import DominoDef (Player, Move, Board, getTurn, staticEval, getMoves, makeMove)


  data GameTree = Tree (Board, [(Move, GameTree)])


  depthGT :: GameTree -> Int
  depthGT gt = aux 0 gt
    where aux d (Tree (b, [])) = d
          aux d (Tree (b, ((m,g2):ms))) = aux (d+1) g2



  createGameTree :: Board -> GameTree
  createGameTree b = Tree (b, map (\m -> (m, createGameTree (makeMove b m))) (getMoves b))


  getBestMove :: Int -> Player -> GameTree -> (Move, Int)
  getBestMove d p (Tree (b, ms)) = getBest (map aux ms)

    where aux (move, gt) = (move, (getPayOffAB d p (-1000000) 1000000 gt))
          getBest (m:ms) = foldl auxcomp m ms
          auxcomp (m1, s1) (m2, s2) = if (s1 >= s2) then (m1, s1) else (m2, s2)



  getPayOffAB :: Int -> Player -> Int -> Int -> GameTree -> Int
  getPayOffAB 0 p alpha beta (Tree (b, ms)) = staticEval b p
  getPayOffAB d p alpha beta (Tree (b, [])) = staticEval b p
  getPayOffAB d p alpha beta (Tree (b, ms)) = searchAB alpha beta ms

        where currTurn = getTurn b

              searchAB alpha beta [] = if (p == currTurn) then alpha else beta
              searchAB alpha beta ((move, gt):ms) = let val = getPayOffAB (d-1) p alpha beta gt in
                                                      if (p == currTurn) then
                                                        if (val >= beta) then val
                                                        else searchAB (max alpha val) beta ms
                                                      else
                                                        if (alpha >= (min beta val)) then val
                                                        else searchAB alpha (min beta val) ms
