module PlayDomino (playDomino) where

  import Array
  import DominoDef
  import GameAI



  depth :: Int
  depth = 10

  playDomino :: IO ()
  playDomino = do putStr ("\n\n\nHCSW JAMAICAN 2-PLAYER DOMINOES\n\nAlpha-Beta search, depth = " ++ (show depth)
                          ++ "\n\n(C) 2004, Doug Hoyte and HCSW\n\nYou are the bottom.\n")
                  b <- createBoard
                  doPlay b


  doPlay :: Board -> IO ()
  doPlay b = do dispBoard b
                if whoWon b /= Nobody then
                  do gameOver b
                     return ()
                  else if getTurn b == A then
                         do (best, payoff) <- do return (getBestMove depth A (createGameTree b))
                            putStr ("\n\n--------My expected payoff with this move is " ++ (show payoff) ++ "-------\n\n")
                            doPlay (makeMove b best)
                            return ()
                       else if getMoves b /= [(-1, -1, -1)] then
                         do pmove <- readMove b
                            doPlay (makeMove b pmove)
                            return ()
                            else do doPlay (makeMove b (-1,-1,-1))
                                    return ()



  gameOver :: Board -> IO ()
  gameOver b = do if (whoWon b == A) then putStr "Game over. The computer won.\n\n"
                    else putStr "Game over. You won.\n\n"



  readMove :: Board -> IO (Int, Int, Int)
  readMove brd@(a,b,s1,s2,p) = do l <- getLine
                                  d <- do return (head (words l))
                                  s <- do return (tail (words l))
                                  (d1, d2) <- do return (if p == A then (a !! ((read d)-1)) else (b !! ((read d)-1)))
                                  if (s == ["l"]) then return (d1, d2, 1)
                                    else return (d1, d2, 2)


  getGoodLine :: IO (String)
  getGoodLine = do l <- getLine
                   wl <- do return (words l)
                   case wl of
                     (a:b:[]) | (b=="l" || b=="r") && ((read a) >= 1 && (read a) <= 14) -> return l
                     _ -> do putStr "Input should be of form '<number> <l|r>'\n"
                             getGoodLine


  dispBoard :: Board -> IO ()
  dispBoard (a, b, s1, s2, p) = do putStr "\n\n"
                                   putStr (if p==A then "OPPONENT --> " else "opponent     ")
                                   putStr (foldl (++) "" (map dispDom a))
                                   putStr "\n\n"
                                   putStr ("    [" ++ (show s1) ++ " ... " ++ (show s2) ++ "]\n")
                                   putStr "\n           "
                                   putStr (dispDomNum b 1)
                                   putStr (if p==B then "YOU      --> " else "you          ")
                                   putStr (foldl (++) "" (map dispDom b))
                                   putStr "\n\n"
                                   putStr (if p==A then "Thinking...\n\n" else "What is your move?\n")
                                   return ()

    where dispDom :: (Int, Int) -> String
          dispDom (d1, d2) = "[" ++ (show d1) ++ "|" ++ (show d2) ++ "]  "

          dispDomNum :: [(Int, Int)] -> Int -> String
          dispDomNum [] x = "\n"
          dispDomNum (d:ds) x = (if (x>9) then "     " else "      ") ++(show x) ++ (dispDomNum ds (x+1))
